# -*- coding: utf-8 -*-
from avaland.utils import test_attr

try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote

import requests
from requests import HTTPError, ConnectionError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Artist, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError
from avaland.music_base import MusicBase
from avaland.search import SearchResult


class WikiSeda(MusicBase):
    __site_name__ = 'WikiSeda'

    _search_url = "http://www.getsongg.com/dapp/?order=top&type=all&page=1&query={query}&lang=en&v=70022"
    _download_url = "http://www.getsongg.com/dapp/"
    _site_url = 'https://www.wikiseda.com/'

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text

    @staticmethod
    def _split_album_title(title):
        if " by " in title:
            return title.split("by")
        if ' - ' in title:
            return title.split("-")
        return title, None

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._search_url.format(query=quote(query)), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to WikiSeda server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to WikiSeda server. (HTTPError)")
        data = res.json()
        musics = []
        albums = []
        artists = []
        for i in data:
            if i['type'] == 'song':
                musics.append(
                    Music(id=int(i["id"]), title=self._reformat(i['songname']), artist=i['artist'], url=i['url'],
                          image=i['poster'], source=WikiSeda, download_url=i['mp3']))
            elif i['type'] == 'artist':
                artists.append(
                    Artist(id=int(i["id"]), full_name=i['artist'], url=i['url'], image=i['poster'], source=WikiSeda))
            elif i['type'] == 'album':
                albums.append(
                    Album(id=i["id"], title=self._reformat(i['album']), artist=self._reformat(i['artist']),
                          url=i['url'], image=i['poster'], source=WikiSeda))

        return SearchResult(musics, albums, artists)

    @test_attr(61)  # Hichkas
    def get_artist(self, artist_id):
        # type: (int) -> SearchResult
        try:
            url = self._download_url + "getnewcases?signer_id={id}&lang=en".format(id=artist_id)
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        albums = []
        data = res.json()
        for i in data['items']:
            if i['type'] == 'song':
                musics.append(
                    Music(id=int(i["id"]), title=self._reformat(i['songname']), artist=i['artist'], url=i['url'],
                          image=i['poster'], source=WikiSeda))
            elif i['type'] == 'album':
                albums.append(
                    Album(id=i["id"], title=self._reformat(i['album']), artist=self._reformat(i['artist']),
                          url=i['url'], image=i['poster'], source=WikiSeda))
        return SearchResult(musics=musics, albums=albums,
                            artists=[Artist(full_name=data['artist'][0]['artist'], image=data['artist'][0]['poster'],
                                            id=int(data['artist'][0]['id']))])

    @test_attr(1751)  # Ya To Ya Hich Kas - Maryam Heydarzadeh
    def get_album(self, album_id):
        # type: (str) -> SearchResult
        try:
            url = self._download_url + "getalbumdetail?id={id}&lang=en".format(id=album_id)
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        data = res.json()['song']
        for i in data['albumtracks']:
            musics.append(
                Music(id=int(i['id']), title=self._reformat(i['songname']), artist=i['artist'], url=i['url'],
                      image=i["poster"], source=WikiSeda, download_link=i['mp3']))
        return SearchResult(musics=musics)

    @test_attr(229662)  # Hich - Reza Bahram
    def get_download_url(self, music_id):
        url = self._download_url + "gettrackdetail?id={id}&lang=en".format(id=music_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to WikiSeda server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to WikiSeda server. (HTTPError)")
        data = res.json()['song'][0]
        return data.get('songname'), data.get('artist'), data.get('mp3')

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



# -*- coding: utf-8 -*-

import math
import os
try:
    import pathlib
except ImportError:
    import pathlib2
import sys
import time

import requests
from requests import HTTPError, ConnectionError

from avaland.exceptions import SourceNetworkError


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%.2f %s" % (s, size_name[i])


class Download:
    def __init__(self, title, artist, url, path=None):
        self.url = url
        self.file_name = title + " - " + artist + ".mp3"
        self.path = os.path.abspath(os.path.expanduser(path)) if path else os.path.join(os.getcwd(), artist)

    def get(self):
        if os.path.isfile(os.path.join(self.path, self.file_name)):
            self.path = os.path.join(self.path, self.file_name)
            return
        pathlib.Path(self.path).mkdir(parents=True, exist_ok=True)
        with open(os.path.join(self.path, self.file_name), "wb") as f:
            start = time.time()
            try:
                req = requests.get(self.url, stream=True)
            except ConnectionError:
                raise SourceNetworkError("Cannot download this music.")
            except HTTPError:
                raise SourceNetworkError("Cannot download this music. (HTTPError)")
            total_length = req.headers.get("content-length")
            if total_length is None:
                f.write(req.content)
            else:
                dl = 0
                total_length = int(total_length)
                print("Downloading: %s" % self.file_name.split(".")[0])
                for data in req.iter_content(chunk_size=4096):
                    dl += len(data)
                    f.write(data)
                    done = int(100 * dl / total_length)
                    percent = " " + str(done) if done < 10 else done
                    sys.stdout.write("\r%s%%|%s%s| (%s / %s) %s/s    " % (
                        percent, "█" * int(done // 3), " " * (int((33 - done // 3))), convert_size(dl),
                        convert_size(total_length), convert_size(int(dl // (time.time() - start))).replace(" ", "")))
                    sys.stdout.flush()
        print()
        self.path = os.path.join(self.path, self.file_name)

    def __repr__(self):
        return "{cls}(url={url}, path={path})".format(cls=Download.__name__, url=self.url, path=self.path)



# -*- coding: utf-8 -*-
from avaland.utils import test_attr

try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote

import requests
import urllib3
from requests import HTTPError, ConnectionError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Artist, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError
from avaland.music_base import MusicBase
from avaland.search import SearchResult

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Navahang(MusicBase):
    __site_name__ = 'Navahang'
    _search_url = "https://navahang.com/main-search.php?q={query}&size=50"
    _api_url = "https://navahang.co/navaapi2/"
    _site_url = 'https://www.navahang.com'

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text

    @staticmethod
    def _split_album_title(title):
        if " by " in title:
            return title.split("by")
        if ' - ' in title:
            return title.split("-")
        return title, None

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.get(self._search_url.format(query=quote(query)), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Navahang server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Navahang server. (HTTPError)")

        data = res.json()

        musics = []
        albums = []
        artists = []
        if 'MP3' in data:
            for i in data['MP3']:
                title, artist = i['title'].split(' – ')
                musics.append(
                    Music(id=int(i["id"]), title=self._reformat(title), artist=artist, url=self._site_url + i['url'],
                          image=i['image'], source=Navahang))

        if 'Artist' in data:
            for i in data['Artist']:
                artists.append(
                    Artist(id=int(i["id"]), full_name=i['title'], url=self._site_url + i['url'], image=i['image'],
                           source=Navahang))

        if 'Album' in data:
            for i in data['Album']:
                try:
                    album, artist = i['title'].split(' – ')
                except:
                    album, artist = i['title'], None
                albums.append(
                    Album(id=int(i["id"]), title=self._reformat(album), artist=self._reformat(artist),
                          url=self._site_url + i['url'], image=i['image'], source=Navahang))

        return SearchResult(musics, albums, artists)

    @test_attr(630620)  # Helen
    def get_artist(self, artist_id):
        url = self._api_url + "GetArtistById?id={id}".format(id=artist_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Navahang server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Navahang server. (HTTPError)")
        musics = []
        albums = []
        data = res.json()
        for i in data['works']:
            if i['category'] == 'MP3':
                musics.append(
                    Music(id=i["post_id"], title=self._reformat(i['song_name']), artist=i['artist_name'],
                          image=i['image'], source=Navahang, download_url=i['high_quality']))
            elif i['category'] == 'Album':
                albums.append(
                    Music(id=i["albumId"], title=self._reformat(i['song']), artist=i['artist'],
                          image=i['playerimage'], source=Navahang))
        return SearchResult(musics=musics, albums=albums, artists=[Artist(full_name=data['name'], id=artist_id, source=Navahang)])

    @test_attr(171108)  # Helen - Negahe To
    def get_album(self, album_id):
        # type: (str) -> SearchResult
        url = self._api_url + "GetPlaylistSongs?playlistId={id}".format(id=album_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Navahang server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Navahang server. (HTTPError)")
        musics = []
        data = res.json()['SONG_LIST']
        for i in data:
            musics.append(
                Music(id=i["post_id"], title=self._reformat(i['song_name']), artist=i['artist_name'],
                      url=res.json()['SHARE_LINK']['link'], image=i['image_Mp3'], source=Navahang))
        return SearchResult(musics=musics)

    @test_attr(198177)  # Arash - Dooset Daram (Ft. Helena)
    def get_download_url(self, music_id):
        url = self._api_url + "GetSingleMediaInfo?media_id={id}".format(id=music_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Navahang server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Navahang server. (HTTPError)")
        data = res.json()[0]
        return data.get('song_name'), data.get('artist_name'), data.get('download')

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



from typing import Tuple

from avaland.download import Download
from avaland.search import SearchResult


class Music:
    id = None  # type: int
    title = None  # type: str
    artist = None  # type: str
    full_title = None  # type: str
    album = None  # type: str
    image = None  # type: str
    url = None  # type: str
    download_url = None  # type: str

    source = None

    def __init__(self, **kwargs):
        for i in kwargs.keys():
            setattr(self, i, kwargs[i].strip() if kwargs[i] and isinstance(kwargs[i], str) else kwargs[i])
        self.full_title = self.title + ' - ' + self.artist if self.artist else self.title

    def get_link(self):
        if self.download_url:
            return self.download_url
        else:
            return self.source({}).get_download_url(self.id)[2]

    def download(self, path=None):
        download_url = getattr(self, "download_url", None)
        if download_url:
            download = Download(self.title, self.artist, download_url, path)
            download.get()
            return download
        return self.source({}).download(self.id, path)

    def to_dict(self):
        return {i: self.__dict__[i] if i != "source" else self.__dict__[i].__name__ for i in self.__dict__}

    def __getitem__(self, item):
        return None

    def __repr__(self):
        return "<{id}: {music}>".format(id=str(self.id),
                                        music=self.title + " - " + self.artist if self.artist else self.title)


class Album:
    id = None  # type: int
    title = None  # type: str
    artist = None  # type: str
    full_title = None  # type: str
    musics = tuple()  # type: Tuple[Music]
    image = None  # type: str

    source = None

    def get_items(self):
        # type: () -> SearchResult
        album = self.source({}).get_album(album_id=self.id)
        self.musics = album.musics
        return album

    def __init__(self, **kwargs):
        for i in kwargs.keys():
            setattr(self, i, kwargs[i].strip() if kwargs[i] and isinstance(kwargs[i], str) else kwargs[i])
        self.full_title = self.title + ' - ' + self.artist if self.artist else self.title

    def to_dict(self):
        dict_data = {i: self.__dict__[i] if i != "source" else self.__dict__[i].__name__ for i in self.__dict__}
        dict_data['musics'] = [i.to_dict() for i in self.musics]
        return dict_data

    def __repr__(self):
        return "<{id}: {album}>".format(id=str(self.id),
                                        album=self.title + " - " + self.artist if self.artist else self.title)


class Artist:
    id = None  # type: int
    full_name = None  # type: str
    musics = None  # type: Tuple[Music]
    albums = None  # type: Tuple[Album]
    image = None  # type: str
    url = None  # type: str

    source = None

    def get_items(self):
        # type: () -> SearchResult
        return self.source({}).get_artist(artist_id=self.id)

    def to_dict(self):
        return {i: self.__dict__[i] if i != "source" else self.__dict__[i].__name__ for i in self.__dict__}

    def __init__(self, **kwargs):
        for i in kwargs.keys():
            setattr(self, i, kwargs[i].strip() if kwargs[i] and isinstance(kwargs[i], str) else kwargs[i])

    def __repr__(self):
        return "<{id}: {name}>".format(id=str(self.id), name=self.full_name)



import time
import traceback
from typing import Type, Dict

from avaland.exceptions import AvalandException, SourceNotRegisteredException
from avaland.music_base import MusicBase
from avaland.search import SearchResult
import multiprocessing as mp


class SourceManager:
    _config = dict()
    _sources = dict()

    def __init__(self):
        pass

    def register(self, source, config=None):
        # type: (Type[MusicBase], dict) -> SourceManager
        if config is None:
            config = {}
        self._sources[source.__site_name__] = source
        self._config[source.__site_name__] = config
        return self

    def __repr__(self):
        return "{cls}(sources=<{sources}>)".format(cls=SourceManager.__name__,
                                                   sources=", ".join(i for i in self._sources.keys()))

    @staticmethod
    def _search(query, source, return_dict):
        try:
            return_dict[type(source).__name__] = source.search(query)
        except AvalandException:
            return_dict[type(source).__name__] = SearchResult(None, None, None)
            print(traceback.format_exc())

    def search(self, query, source=None):
        # type: (str, Type[MusicBase]) -> Dict[str, SearchResult]
        sources_search = dict()
        if source:
            if source.__site_name__ not in self._sources:
                raise SourceNotRegisteredException("\"" + source.__site_name__ + "\" is not a registered source.")
            try:
                sources_search[source.__site_name__] = source(self._config[source.__site_name__]).search(query)
            except AvalandException:
                sources_search[source.__site_name__] = SearchResult(None, None, None)
                print(traceback.format_exc())
        else:
            multi_manager = mp.Manager()
            sources_search = multi_manager.dict()
            jobs = []
            for i in self._sources.keys():
                source = self._sources[i](self._config[self._sources[i].__site_name__])
                p = mp.Process(target=self._search, args=(query, source, sources_search))
                jobs.append(p)
                p.start()
            for proc in jobs:
                proc.join()
        return sources_search



from avaland.data_types import Music
from avaland.download import Download
from avaland.search import SearchResult


class MusicBase(object):
    __site_name__ = ''
    config = dict()
    testing = False

    def __init__(self, config):
        self.config = config

    def search(self, query):
        # type: (str) -> SearchResult
        raise NotImplementedError

    def get_artist(self, artist_id):
        # type: (str) -> SearchResult
        raise NotImplementedError

    def get_album(self, album_id):
        # type: (str) -> SearchResult
        raise NotImplementedError

    def get_download_url(self, music):
        # type: (Music) -> str
        raise NotImplementedError

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        raise NotImplementedError



# -*- coding: utf-8 -*-
from avaland.utils import test_attr

try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote

import requests
from requests import HTTPError, ConnectionError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError
from avaland.music_base import MusicBase
from avaland.search import SearchResult


class RapFarsi(MusicBase):
    __site_name__ = 'RapFarsi'

    _search_url = "http://51.89.98.91:3004/S/search/{query}"

    _headers = {"User-Agent": "okhttp/3.4.1",
                "Accept-Encoding": "gzip",
                "Content-Type": "application/x-www-form-urlencoded"}

    _download_url = "http://51.89.98.91:3004/S/{id}"
    _site_url = 'https://www.rapfarsi.co'

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._search_url.format(query=quote(query)), headers=self._headers,
                                timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to RapFarsi server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to RapFarsi server. (HTTPError)")
        data = res.json()
        musics = []
        albums = []
        artists = []
        for i in data:
            if i['type'] == 'single':
                musics.append(Music(id=i["slug"], title=self._reformat(i['title']), artist=i['artistTitle'],
                                    url=self._site_url + i['link'], image=i['cover']['full'], source=RapFarsi,
                                    download_link=i['track320']['link'] if i['track320'] else i['track128']['link'] if
                                    'track128' in i and i['track128'] else None))
            elif i['type'] == 'album':
                albums.append(
                    Album(id=i["slug"], title=self._reformat(i['title']), artist=self._reformat(i['artistTitle']),
                          url=self._site_url + i['link'], image=i['cover']['full'], source=RapFarsi))

        return SearchResult(musics, albums, artists)

    @test_attr("hichkas-dastasho-mosht-karde")  # Dastasho Mosht Karde - Hichkas
    def get_download_url(self, music_id):
        url = self._download_url.format(id=music_id)
        try:
            res = requests.post(url, headers=self._headers, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to RapFarsi server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to RapFarsi server. (HTTPError)")
        data = res.json()['Post']
        return data.get('title'), data.get('artistTitle'), data['track320']['link']

    @test_attr(0)  # nothing!
    def get_artist(self, artist_id):
        return SearchResult(None, None, None)

    @test_attr("hichkas-mojaz")  # Mojaz - Hichkas
    def get_album(self, album_id):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._download_url.format(id=album_id), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        data = res.json()['Post']
        for i in data['albumList']:
            artist, title = data['artistTitle'], i['title']
            musics.append(
                Music(id=i['id'], title=self._reformat(title), artist=artist, url=self._site_url + data['link'],
                      image=i["cover"]['full'], source=RapFarsi,
                      download_link=i['track320']['link'] if i['track320'] else i['track128']['link']))
        return SearchResult(musics=musics)

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



# -*- coding: utf-8 -*-

from xml.etree import ElementTree

import requests
from requests.exceptions import ConnectionError, HTTPError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Artist, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError, SourceException
from avaland.music_base import MusicBase
from avaland.search import SearchResult
from avaland.utils import test_attr


class Bia2(MusicBase):
    __site_name__ = 'Bia2'

    _search_url = "http://api.swiftype.com/api/v1/public/engines/search.json"
    _download_url = "http://cdn.biatik.com/api/android/2"
    _site_url = 'https://www.bia2.com/'
    _engine_key = "TxEmaszMvyJTZhnHaDBq"

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text.replace("on Bia2", "")

    @staticmethod
    def _split_album_title(title):
        if " by " in title:
            return title.split("by")
        if ' - ' in title:
            return title.split("-")
        return title, None

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._search_url,
                                data={"engine_key": self._engine_key, "q": query, "per_page": 50}, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to bia2music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to bia2music server. (HTTPError)")

        if res.status_code == 404:
            raise SourceException("Invalid response data from Bia2")
        data = res.json()['records']['page']
        musics = []
        albums = []
        artists = []
        for i in data:
            if i['type'] == 'music':
                musics.append(
                    Music(id=i["music_id"], title=self._reformat(i['title']), artist=i['artist'], url=i['url'],
                          image=i['image'], source=Bia2))
            elif i['type'] == 'artist':
                artists.append(
                    Artist(id=i["artist_id"], full_name=i['title'], url=i['url'], image=i['image'], source=Bia2))
            elif i['type'] == '':
                title = self._split_album_title(i['title'])
                albums.append(
                    Album(id=i["url"].split('/')[-1], title=self._reformat(title[0]), artist=self._reformat(title[1]),
                          url=i['url'], image=i['image'], source=Bia2))

        return SearchResult(musics, albums, artists)

    @test_attr(548)  # Helen
    def get_artist(self, artist_id):
        # type: (int) -> SearchResult
        url = self._download_url + '/artist.xml?id={}'.format(artist_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to bia2music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to bia2music server. (HTTPError)")
        data = ElementTree.fromstring(res.content)[0]
        musics = []
        albums = []
        for i in range(len(data)):
            if data[i].tag == 'albums':
                for j in data[i]:
                    albums.append(
                        Album(id=int(j.get('id')), title=self._reformat(j.get('title')),
                              artist=self._reformat(data.get("name")), image=j.get('cover'), source=Bia2))

            if data[i].tag == 'singles':
                for j in data[i]:
                    musics.append(
                        Music(id=int(j.get('id')), title=self._reformat(j.get('title')), artist=data.get("name"),
                              url=data.get('share_url'), image=j.get('cover'), source=Bia2))
        return SearchResult(musics=musics, albums=albums, artists=[Artist(full_name=data.get("name"), id=artist_id,
                                                                          image=data.get("cover"), source=Bia2)])

    @test_attr(1267)  # Moon And Star - Helen
    def get_album(self, album_id):
        # type: (int) -> SearchResult
        url = self._download_url + '/album.xml?album={}'.format(album_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to bia2music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to bia2music server. (HTTPError)")
        data = ElementTree.fromstring(res.content)[0]
        musics = []
        for i in data:
            musics.append(
                Music(id=int(i.get("id")), title=self._reformat(i.get("title")), artist=data.get('artist'),
                      url=data.get('share_url'), image=data.get('cover'), source=Bia2))
        return SearchResult(musics=musics)

    @test_attr(42271)  # Shomine - Helen
    def get_download_url(self, music_id):
        url = self._download_url + '/music.xml?id={}'.format(music_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to bia2music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to bia2music server. (HTTPError)")
        data = ElementTree.fromstring(res.content)[0]
        return data.get('Title'), data.get('Artist'), data.get('mp3')

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



class SearchResult(object):

    def __init__(self, musics=None, albums=None, artists=None):
        self.musics = tuple(musics) if musics else tuple()
        self.albums = tuple(albums) if albums else tuple()
        self.artists = tuple(artists) if artists else tuple()
        self.record_count = len(self.musics) + len(self.albums) + len(self.artists)

    def to_dict(self):
        return {
            "musics": [i.to_dict() for i in self.musics],
            "albums": [i.to_dict() for i in self.albums],
            "artists": [i.to_dict() for i in self.artists],
        }

    def __repr__(self):
        musics = self.musics[0].full_title + ',...' if len(self.musics) > 0 else None
        albums = self.albums[0].full_title + ',...' if len(self.albums) > 0 else None
        artists = self.artists[0].full_name + ',...' if len(self.artists) > 0 else None
        return "{cls}(musics=<{musics}>, albums=<{albums}>, artists=<{artists}>)".format(cls=SearchResult.__name__,
                                                                                         musics=musics,
                                                                                         albums=albums,
                                                                                         artists=artists)



class AvalandException(Exception):
    pass


class SourceNotRegisteredException(AvalandException):
    pass


class SourceException(AvalandException):
    pass


class SourceNetworkError(SourceException):
    pass


class GatewayInvalidError(SourceException):
    pass



# -*- coding: utf-8 -*-
from avaland.utils import test_attr

try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote

import requests
from requests import HTTPError, ConnectionError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Artist, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError
from avaland.music_base import MusicBase
from avaland.search import SearchResult


class RadioJavan(MusicBase):
    __site_name__ = 'RadioJavan'

    _search_url = "https://api-rjvn.app/api2/search?query={query}"
    _download_url = "https://api-rjvn.app/api2/"
    _site_url = 'https://www.radiojavan.com'

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text

    @staticmethod
    def _split_album_title(title):
        if " by " in title:
            return title.split("by")
        if ' - ' in title:
            return title.split("-")
        return title, None

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.get(self._search_url.format(query=quote(query)), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to RadioJavan server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to RadioJavan server. (HTTPError)")
        data = res.json()
        musics = []
        albums = []
        artists = []
        if 'mp3s' in data:
            for i in data['mp3s']:
                musics.append(
                    Music(id=int(i["id"]), title=i['song'], artist=i['artist'], url=i['share_link'],
                          image=i['photo'], source=RadioJavan, download_url=i['link']))

        if 'artists' in data:
            for i in data['artists']:
                artists.append(
                    Artist(id=quote(i['name']), full_name=i['name'], image=i['photo'], source=RadioJavan))

        if 'albums' in data:
            for i in data['albums']:
                albums.append(
                    Album(id=int(i["id"]), title=self._reformat(i['album_album']),
                          artist=self._reformat(i['album_artist']), url=i['share_link'], image=i['photo'],
                          source=RadioJavan))

        return SearchResult(musics, albums, artists)

    @test_attr("Hichkas")  # Hichkas
    def get_artist(self, artist_id):
        # type: (str) -> SearchResult
        try:
            res = requests.get(self._download_url + "artist?query={id}".format(id=artist_id), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        albums = []
        data = res.json()
        if 'mp3s' in data:
            for i in data['mp3s']:
                musics.append(
                    Music(id=int(i["id"]), title=i['song'], artist=i['artist'], url=i['share_link'],
                          image=i['photo'], source=RadioJavan, download_url=i['link']))

        if 'albums' in data:
            for i in data['albums']:
                albums.append(
                    Album(id=int(i["album_id"]), title=self._reformat(i['album_album']),
                          artist=self._reformat(i['album_artist']), url=i['album']['share_link'], image=i['photo'],
                          source=RadioJavan))

        return SearchResult(musics=musics, albums=albums, artists=[Artist(full_name=data['query'], id=artist_id, source=RadioJavan)])

    @test_attr(44893)  # Jangale Asfalt - Hichkas
    def get_album(self, album_id):
        # type: (int) -> SearchResult
        try:
            res = requests.get(self._download_url + "mp3?id={id}".format(id=album_id), timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        data = res.json()
        albums = []
        artist = [Artist(full_name=data['artist'], id=quote(data['artist']).lower())]
        if 'album_tracks' in data:
            albums.append(Album(artist=data['album_artist'], title=data['album_album']))
            for i in data['album_tracks']:
                if i['type'] == 'mp3':
                    musics.append(
                        Music(id=i['id'], title=self._reformat(i['song']), artist=i['artist'],
                              url=i['share_link'], image=i["photo"], source=RadioJavan, download_link=i['link']))
        else:
            if data['type'] == 'mp3':
                musics.append(
                    Music(id=data['id'], title=self._reformat(data['song']), artist=data['artist'],
                          url=data['share_link'], image=data["photo"], source=RadioJavan, download_link=data['link']))
        return SearchResult(musics=musics, albums=albums, artists=artist)

    @test_attr(88782)  # Hich - Reza Bahram
    def get_download_url(self, music_id):
        url = self._download_url + "mp3?id={id}".format(id=music_id)
        try:
            res = requests.get(url, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to RadioJavan server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to RadioJavan server. (HTTPError)")
        data = res.json()
        return data.get('song'), data.get('artist'), data.get('link')

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



from .data_types import Album, Artist, Music
from .exceptions import (AvalandException, SourceException, SourceNetworkError, GatewayInvalidError,
                         SourceNotRegisteredException)
from .manager import SourceManager
from .music_base import MusicBase

__version__ = '0.0.1'



# -*- coding: utf-8 -*-
import json

import requests
import urllib3
from requests import HTTPError, ConnectionError

from avaland.config import MAX_TIME_OUT
from avaland.data_types import Music, Album
from avaland.download import Download
from avaland.exceptions import SourceNetworkError
from avaland.music_base import MusicBase
from avaland.search import SearchResult
from avaland.utils import test_attr

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Nex1(MusicBase):
    __site_name__ = 'Nex1Music'

    _search_url = "https://apin1mservice.com/WebService/search.php"
    _download_url = "https://apin1mservice.com/WebService/music-more.php"
    _site_url = 'https://nex1music.ir'

    def __init__(self, config):
        MusicBase.__init__(self, config)

    @staticmethod
    def _reformat(text):
        return text

    @staticmethod
    def _split_album_title(title):
        if " by " in title:
            return title.split("by")
        if ' - ' in title:
            return title.split("-")
        return title, None

    @test_attr("hello")
    def search(self, query):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._search_url, data={"text": query}, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Nex1 Music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Nex1 Music server. (HTTPError)")
        data = res.json()
        musics = []
        albums = []
        artists = []
        for i in data:
            if 'type' in i and i['type'] == 'تک آهنگ':
                musics.append(
                    Music(id=int(i["id"]), title=self._reformat(i['tracken']), artist=i['artisten'], url=None,
                          image=i['image'], source=Nex1))

            elif 'type' in i and i['type'] == 'آلبوم':
                albums.append(
                    Album(id=i["id"], title=self._reformat(i['tracken']), artist=self._reformat(i['artisten']),
                          url=None, image=i['image'], source=Nex1))

        return SearchResult(musics, albums, artists)

    @test_attr(12715)  # Hel Hele - Amir Masih
    def get_download_url(self, music_id):
        url = self._download_url
        try:
            res = requests.post(url, data={"post_id": music_id}, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Nex1 Music server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Nex1 Music server. (HTTPError)")
        data = res.json()
        return data.get('TrackEn'), data.get('ArtistEn'), data.get('Music320')

    @test_attr(0)  # nothing!
    def get_artist(self, artist_id):
        return SearchResult(None, None, None)

    @test_attr(12792)  # Mojaz - Hichkas
    def get_album(self, album_id):
        # type: (str) -> SearchResult
        try:
            res = requests.post(self._download_url, data={"post_id": album_id}, timeout=MAX_TIME_OUT)
        except ConnectionError:
            raise SourceNetworkError("Cannot connect to Next1 server.")
        except HTTPError:
            raise SourceNetworkError("Cannot connect to Next1 server. (HTTPError)")
        musics = []
        data = json.loads(res.json()['AlbumTrackList'])
        for i in data:
            musics.append(
                Music(id=None, title=self._reformat(i['album_track_name_en']), artist=res.json()["ArtistEn"],
                      url=res.json()['PostUrl'], image=res.json()["Image"], source=Nex1,
                      download_url=i['album_track_link_320']))
        return SearchResult(musics=musics)

    def download(self, music_id, path=None):
        # type: (int, str) -> Download
        title, artist, url = self.get_download_url(music_id)
        download = Download(title, artist, url, path)
        download.get()
        return download



import inspect
import unittest

from avaland import sources
from avaland import MusicBase
from avaland.search import SearchResult


def test_search(self):
    query = self.source.search.test
    source = self.source({}).search(query)
    self.assertIsInstance(source, SearchResult)


def test_artist_id(self):
    artist = self.source.get_artist.test
    source = self.source({}).get_artist(artist)
    self.assertIsInstance(source, SearchResult)


def test_album_id(self):
    album = self.source.get_album.test
    source = self.source({}).get_album(album)
    self.assertIsInstance(source, SearchResult)


def test_download_url(self):
    music_id = self.source.get_download_url.test
    data = self.source({}).get_download_url(music_id)
    self.assertEqual(len(data), 3)


def _create_class(name, obj):
    _class = type(name + "Test", (unittest.TestCase,), {
        "source": obj,
        "test_search": test_search,
        "test_artist": test_artist_id,
        "test_album": test_album_id,
        "test_download": test_download_url
    })
    return _class


class_members = inspect.getmembers(sources, inspect.isclass)

for i in class_members:
    if issubclass(i[1], MusicBase) and i[1] != MusicBase:
        globals()[i[0]] = _create_class(i[0], i[1])





from setuptools import setup, find_packages
from os import path

current_dir = path.abspath(path.dirname(__file__))

with open(path.join(current_dir, 'README.md')) as f:
    description = f.read()
    
    
setup(
    name='avaland',
    version='0.0.1',
    description='Avaland is a music downloader to download music from many sources.',
    long_description=description,
    long_description_content_type='text/markdown',
    url='https://github.com/Dragon-Born/avaland',
    download_url = 'https://github.com/Dragon-Born/avaland/archive/v0.1.tar.gz',
    keywords=[
        'Avaland',
        'Persian music', 'Persian music downloader', 'Persian music api',
        'Iranian music', 'Iranian music downloader', 'Iranian music api',
    ],
    license='MIT',
    test_suite="test",
    author='Arian Amiramjadi',
    author_email='me@arian.lol',
    packages=find_packages(),
    install_requires=['requests', 'pathlib2', 'typing'],
    scripts=['bin/avaland'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Environment :: Console',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
